/****************************************************************************
** Meta object code from reading C++ file 'smartfailure.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../plasma-disks-6.7.4/src/smartfailure.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smartfailure.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5SMARTE_t {};
} // unnamed namespace

template <> constexpr inline auto SMART::qt_create_metaobjectdata<qt_meta_tag_ZN5SMARTE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SMART",
        "Failure",
        "None",
        "CmdLineParse",
        "DeviceOpen",
        "InternalCommand",
        "Disk",
        "Prefail",
        "PastPrefail",
        "ErrorsRecorded",
        "SelfTestErrors"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Failure'
        QtMocHelpers::EnumData<Failure>(1, 1, QMC::EnumIsScoped).add({
            {    2, Failure::None },
            {    3, Failure::CmdLineParse },
            {    4, Failure::DeviceOpen },
            {    5, Failure::InternalCommand },
            {    6, Failure::Disk },
            {    7, Failure::Prefail },
            {    8, Failure::PastPrefail },
            {    9, Failure::ErrorsRecorded },
            {   10, Failure::SelfTestErrors },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN5SMARTE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN5SMARTE =
    SMART::qt_create_metaobjectdata<qt_meta_tag_ZN5SMARTE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN5SMARTE =
    qt_staticMetaObjectContent_ZN5SMARTE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN5SMARTE =
    qt_staticMetaObjectContent_ZN5SMARTE.relocatingData;

Q_CONSTINIT const QMetaObject SMART::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN5SMARTE.stringdata,
    qt_staticMetaObjectStaticContent_ZN5SMARTE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN5SMARTE.metaTypes,
    nullptr
} };

QT_WARNING_POP
