/****************************************************************************
** Meta object code from reading C++ file 'dbusobjectmanagerserver.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../plasma-disks-6.7.4/src/dbusobjectmanagerserver.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbusobjectmanagerserver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN24KDBusObjectManagerServerE_t {};
} // unnamed namespace

template <> constexpr inline auto KDBusObjectManagerServer::qt_create_metaobjectdata<qt_meta_tag_ZN24KDBusObjectManagerServerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KDBusObjectManagerServer",
        "D-Bus Interface",
        "org.freedesktop.DBus.ObjectManager",
        "InterfacesAdded",
        "",
        "QDBusObjectPath",
        "path",
        "KDBusObjectManagerInterfacePropertiesMap",
        "map",
        "InterfacesRemoved",
        "KDBusObjectManagerInterfaceList",
        "list",
        "GetManagedObjects",
        "KDBusObjectManagerObjectPathInterfacePropertiesMap"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'InterfacesAdded'
        QtMocHelpers::SignalData<void(QDBusObjectPath, KDBusObjectManagerInterfacePropertiesMap)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'InterfacesRemoved'
        QtMocHelpers::SignalData<void(QDBusObjectPath, KDBusObjectManagerInterfaceList)>(9, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 10, 11 },
        }}),
        // Slot 'GetManagedObjects'
        QtMocHelpers::SlotData<KDBusObjectManagerObjectPathInterfacePropertiesMap()>(12, 4, QMC::AccessPublic, 0x80000000 | 13),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<KDBusObjectManagerServer, qt_meta_tag_ZN24KDBusObjectManagerServerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject KDBusObjectManagerServer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24KDBusObjectManagerServerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24KDBusObjectManagerServerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24KDBusObjectManagerServerE_t>.metaTypes,
    nullptr
} };

void KDBusObjectManagerServer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KDBusObjectManagerServer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->InterfacesAdded((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KDBusObjectManagerInterfacePropertiesMap>>(_a[2]))); break;
        case 1: _t->InterfacesRemoved((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KDBusObjectManagerInterfaceList>>(_a[2]))); break;
        case 2: { KDBusObjectManagerObjectPathInterfacePropertiesMap _r = _t->GetManagedObjects();
            if (_a[0]) *reinterpret_cast<KDBusObjectManagerObjectPathInterfacePropertiesMap*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KDBusObjectManagerInterfacePropertiesMap >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KDBusObjectManagerInterfaceList >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KDBusObjectManagerServer::*)(QDBusObjectPath , KDBusObjectManagerInterfacePropertiesMap )>(_a, &KDBusObjectManagerServer::InterfacesAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KDBusObjectManagerServer::*)(QDBusObjectPath , KDBusObjectManagerInterfaceList )>(_a, &KDBusObjectManagerServer::InterfacesRemoved, 1))
            return;
    }
}

const QMetaObject *KDBusObjectManagerServer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDBusObjectManagerServer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24KDBusObjectManagerServerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KDBusObjectManagerServer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void KDBusObjectManagerServer::InterfacesAdded(QDBusObjectPath _t1, KDBusObjectManagerInterfacePropertiesMap _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void KDBusObjectManagerServer::InterfacesRemoved(QDBusObjectPath _t1, KDBusObjectManagerInterfaceList _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
