/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kded.smart.Device.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef ORG_KDE_KDED_SMART_DEVICE_H
#define ORG_KDE_KDED_SMART_DEVICE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kded.smart.Device
 */
class OrgKdeKdedSmartDeviceInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.kded.smart.Device"; }

public:
    OrgKdeKdedSmartDeviceInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKdedSmartDeviceInterface();

    Q_PROPERTY(QString advancedReport READ advancedReport)
    inline QString advancedReport() const
    { return qvariant_cast< QString >(property("advancedReport")); }

    Q_PROPERTY(bool failed READ failed WRITE setFailed)
    inline bool failed() const
    { return qvariant_cast< bool >(property("failed")); }
    inline void setFailed(bool value)
    { setProperty("failed", QVariant::fromValue(value)); }

    Q_PROPERTY(bool ignore READ ignore WRITE setIgnore)
    inline bool ignore() const
    { return qvariant_cast< bool >(property("ignore")); }
    inline void setIgnore(bool value)
    { setProperty("ignore", QVariant::fromValue(value)); }

    Q_PROPERTY(QStringList instabilities READ instabilities WRITE setInstabilities)
    inline QStringList instabilities() const
    { return qvariant_cast< QStringList >(property("instabilities")); }
    inline void setInstabilities(const QStringList &value)
    { setProperty("instabilities", QVariant::fromValue(value)); }

    Q_PROPERTY(QString path READ path)
    inline QString path() const
    { return qvariant_cast< QString >(property("path")); }

    Q_PROPERTY(QString product READ product)
    inline QString product() const
    { return qvariant_cast< QString >(property("product")); }

    Q_PROPERTY(QString udi READ udi)
    inline QString udi() const
    { return qvariant_cast< QString >(property("udi")); }

public Q_SLOTS: // METHODS
Q_SIGNALS: // SIGNALS
    void advancedReportChanged();
    void failedChanged();
    void ignoreChanged();
    void instabilitiesChanged();
};

namespace org {
  namespace kde {
    namespace kded {
      namespace smart {
        using Device = ::OrgKdeKdedSmartDeviceInterface;
      }
    }
  }
}

#endif
